////////////////////////////////////////////////////////////////////////////////
//
//  ****************************************************************************
//  * Project   : Bezier
//  * Unit Name : uBezierMain
//  * Purpose   : ,      
//  *           :    PolyBezier, PolyBezierTo
//  *           :   - ,      :)
//  * Author    :  (Rouse_) 
//  * Copyright :  Fangorn Wizards Lab 1998 - 2007
//  * Version   : 1.00
//  * Home Page : http://rouse.drkb.ru
//  ****************************************************************************
//

unit uBezierMain;

interface

uses
  Windows, Messages, SysUtils, Classes,
  Graphics, Controls, Forms,  Dialogs;

type
  TBezierDemo = class(TForm)
    procedure FormMouseUp(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure FormMouseMove(Sender: TObject; Shift: TShiftState; X, Y: Integer);
    procedure FormMouseDown(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure FormCreate(Sender: TObject);
    procedure FormPaint(Sender: TObject);
  private
    EditedPoint: PPoint;
    A, A1, B1, B: TPoint;
  public
    function GetBezierPoint(P0, P1, P2, P3: TPoint;
      Resolution, Index: Integer): TPoint;
    procedure DrawBezier;
  end;

var
  BezierDemo: TBezierDemo;

implementation

{$R *.dfm}

//        4  
// =============================================================================
procedure TBezierDemo.DrawBezier;
const
  Resolution = 500;
var
  I: Integer;
  P: TPoint;
begin
  //        
  Canvas.MoveTo(A.X, A.Y);
  Canvas.LineTo(A1.X, A1.Y);
  //    
  Canvas.Pen.Width := 3;
  Canvas.Ellipse(A.X - 4, A.Y - 4, A.X + 4, A.Y + 4);
  Canvas.Ellipse(A1.X - 4, A1.Y - 4, A1.X + 4, A1.Y + 4);
  Canvas.Pen.Width := 1;
  //        B
  Canvas.MoveTo(B.X, B.Y);
  Canvas.LineTo(B1.X, B1.Y);
  //    
  Canvas.Pen.Width := 3;
  Canvas.Ellipse(B1.X - 4, B1.Y - 4, B1.X + 4, B1.Y + 4);
  Canvas.Ellipse(B.X - 4, B.Y - 4, B.X + 4, B.Y + 4);
  Canvas.Pen.Width := 1;

  //      
  for I := 0 to Resolution - 1 do
  begin
    //   
    P := GetBezierPoint(A, A1, B1, B, Resolution, I);
    //   
    Canvas.Pen.Color := clRed;
    Canvas.Ellipse(P.X, P.Y, P.X + 2, P.Y + 2);
    Canvas.Pen.Color := clBlack;
  end;
end;

//   
// =============================================================================
procedure TBezierDemo.FormCreate(Sender: TObject);
begin
  DoubleBuffered := True;
  EditedPoint := nil;
  A := Point(303, 10);
  A1 := Point(53, 159);
  B1 := Point(404, 236);
  B := Point(47, 268);
end;

//     
// =============================================================================
procedure TBezierDemo.FormMouseDown(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);

  //       
  //          
  function MakeRectFromPoint(const Value: TPoint): TRect;
  begin
    Result.Left := Value.X - 6;
    Result.Top := Value.Y - 6;
    Result.Right := Result.Left + 12;
    Result.Bottom := Result.Top + 12;
  end;

begin
  if not (ssLeft in Shift) then Exit;
  //         ?
  if PtInRect(MakeRectFromPoint(A), Point(X, Y)) then
  begin
    // ,   
    EditedPoint := @A;
    Exit;
  end;
  //         1?
  if PtInRect(MakeRectFromPoint(A1), Point(X, Y)) then
  begin
    // ,   
    EditedPoint := @A1;
    Exit;
  end;
  //         1?
  if PtInRect(MakeRectFromPoint(B1), Point(X, Y)) then
  begin
    // ,   
    EditedPoint := @B1;
    Exit;
  end;
  //         ?
  if PtInRect(MakeRectFromPoint(B), Point(X, Y)) then
    // ,   
    EditedPoint := @B;
end;

//      
//     
// =============================================================================
procedure TBezierDemo.FormMouseMove(Sender: TObject; Shift: TShiftState; X,
  Y: Integer);
begin
  if not (ssLeft in Shift) then Exit;
  if EditedPoint = nil then Exit;
  EditedPoint^.X := X;
  EditedPoint^.Y := Y;
  Repaint;
end;

//   , 
// =============================================================================
procedure TBezierDemo.FormMouseUp(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
begin
  EditedPoint := nil;
end;

//
// =============================================================================
procedure TBezierDemo.FormPaint(Sender: TObject);
begin
  DrawBezier;
end;

//        ,  
//           
//   :
//  P0, P1, P2, P3 -     
//  Resolution -   ( - )
//  Index -  ,    
// =============================================================================
function TBezierDemo.GetBezierPoint(P0, P1, P2, P3: TPoint;
  Resolution, Index: Integer): TPoint;
var
  T, C0, C1, C2, C3: Extended;
begin
  //      
  // (   )
  //   
  // B(t) = (1-t)^3P0 + 3t(1-t)^2P1 + 3t^2(1-t)P2 + P3t^3 ,
  //  t     0  1
  T := Index / Resolution;
  C3 := T * T * T;
  C0 := C3 * -1 + T * T * 3 + T * -3 + 1;
  C1 := C3 * 3 + T * T * -6 + T * 3;
  C2 := C3 * -3 + T * T * 3;
  Result.X := Trunc(C0 * P0.X + C1 * P1.X + C2 * P2.X + C3 * P3.X);
  Result.Y := Trunc(C0 * P0.Y + C1 * P1.Y + C2 * P2.Y + C3 * P3.Y);
end;

end.
